//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: pibeta.hh 2006/01/28 15:56:58 Volnykh V.P. Exp $
// GEANT4 tag $Name: geant4-08-00 $
//
//
// -------------------------------------------------------------------
// Class description:
// Example of analysis in a simulation application (histograms, ntuples etc.)
// This class follows the singleton design pattern; 
// it is responsible for the analysis management and algorithms 
// Histograms are compliant with AIDA 1.0 (the header files are
// copies from the Anaphe Interfaces directory), except for the usage of
// IHistoManager which is Anaphe/Lizard-specific
// For ntuples, for which the AIDA interfaces and compliant implementations
// are still in progress at the time of the current Geant4 release, 
// an implementation with Anaphe/Lizard is shown
// Other implementations specific to an analysis system are possible too
// (see, for instance, JAS and OpenScientist documentation from the links
// in http://aida.freehep.org/)
// The implementation of the usage of ntuples shown in this example
// is expected to change in future Geant4 releases, when AIDA interfaces 
// and related implementations would be available
// Further documentation is available from: http://www.ge.infn.it/geant4/lowE/
//                                          http://aida.freehep.org/
//                                          http://cern.ch/anaphe/

// -------------------------------------------------------------------

#ifndef G4PROCESSTESTANALYSIS_HH
#define G4PROCESSTESTANALYSIS_HH

#include "globals.hh"
#include "G4ios.hh"

#ifdef G4ANALYSIS_USE
#include "AIDA/IAnalysisFactory.h"

#include "AIDA/ITreeFactory.h"
#include "AIDA/ITree.h"

#include "AIDA/IHistogramFactory.h"
#include "AIDA/IHistogram1D.h"
#include "AIDA/IHistogram2D.h"
#include "AIDA/IHistogram3D.h"

#include "AIDA/IPlotterFactory.h"
#include "AIDA/IPlotterRegion.h"
#include "AIDA/IPlotter.h"

#include "AIDA/ITupleFactory.h"
#include "AIDA/ITuple.h"

#include "AIDA/IManagedObject.h"

// Histogramming from AIDA 
class IAnalysisFactory;
class ITree;
class IHistogramFactory;
class ITupleFactory;
class ITuple;
#endif

//class G4Track;

class G4Step;

class PSIAnalysis
{
public:

  virtual ~PSIAnalysis();

  void book();
  
  void finish();

  //fill histograms with data from PSISteppingAction
  void analyseStepping(const G4Step* aStep);

   //fill histograms with data from PSIPrimarygeneratorAction
  void analysePrimaryGenerator(G4double energy,G4double xx,G4double yy,G4double zz);
  
  //fill histograms witn data in BeginofEvent
  void analyseBeginofEvent(G4int evtNbegin);
  
  //fill histograms witn data in EndofEvent
  void analyseEndofEvent(G4int evtNb);
   
  //method to call to create an instance of this class
  static PSIAnalysis* getInstance();

private:

  //private constructor in order to create a singleton
  PSIAnalysis();

  std::vector<G4double>* gunParticleEnergies;

  //Instance for singleton implementation this is the returned
  static PSIAnalysis* instance;

  // analysis data members

// #ifdef G4ANALYSIS_USE_PLOTTER
//   void plotAll();
// #endif


#ifdef G4ANALYSIS_USE
  AIDA::IAnalysisFactory  *analysisFactory;
  AIDA::ITree             *tree;
  AIDA::ITree* treeDet;
  AIDA::IHistogramFactory *histoFactory;
  AIDA::ITupleFactory     *tupleFactory;
// #ifdef G4ANALYSIS_USE_PLOTTER
//   AIDA::IPlotterFactory   *plotterFactory;
//   AIDA::IPlotter          *plotter;
// #endif
#endif


  G4String asciiFileName;
  G4String histFileName;
  G4String histFileType;
//  AIDA::IHistogram1D *h1;
//  AIDA::IHistogram1D *h2;
//  AIDA::IHistogram1D *h3;
//  AIDA::IHistogram1D *h4;
//  AIDA::IHistogram2D *h4;
  AIDA::ITuple * ntuple100;
  AIDA::ITuple * ntuple101;
//  AIDA::ITuple * ntuple102;
//  AIDA::ITuple * ntuple103;
  //  std::ofstream asciiFile;

};

#endif 
