//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// $Id: pibeta.cc,v 1.25 2005/12/06 10:48:08 volnykh Exp $
// GEANT4 tag $Name: geant4-08-00 $
//
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//
// Description: Continuous Process G4Cerenkov
//              and RestDiscrete Process G4Scintillation
//              -- Generation Cerenkov Photons --
//              -- Generation Scintillation Photons --
//              -- Transport of optical Photons --
//
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "G4RunManager.hh"
#include "G4UImanager.hh"
#include "G4UIterminal.hh"
#include "G4UItcsh.hh"

#include "G4ios.hh"
#include <stdlib.h>

#ifdef G4UI_USE_XM
#include "G4UIXm.hh"
#endif

#ifdef G4UI_USE_WIN32
#include "G4UIWin32.hh"
#endif

#include "Randomize.hh"

#include "PSIDetectorConstruction.hh"
#include "PSIAnalysis.hh"
#include "PSIPhysicsList.hh"
#include "PSIPrimaryGeneratorAction.hh"
#include "PSIRunAction.hh"
#include "PSIEventAction.hh"
//#include "PSIStackingAction.hh"
#include "PSISteppingAction.hh"
#include "PSISteppingVerbose.hh"

#ifdef G4VIS_USE
#include "G4VisExecutive.hh"
#endif

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

int main(int argc,char** argv) {

  // Seed the random number generator manually
  //G4long myseed = 345354;
  //CLHEP::HepRandom::setTheSeed(myseed);
  
    // choose the Random engine
  CLHEP::HepRandom::setTheEngine(new CLHEP::RanecuEngine);
  
  //my Verbose output class
  G4VSteppingVerbose::SetInstance(new PSISteppingVerbose);
  
  // Run manager
  G4RunManager* runManager = new G4RunManager;

  // UserInitialization classes - mandatory
  //runManager-> SetUserInitialization(new PSIDetectorConstruction);
  //runManager-> SetUserInitialization(new PSIPhysicsList);
  
    // set mandatory initialization classes
  PSIDetectorConstruction* detector = new PSIDetectorConstruction;
  runManager->SetUserInitialization(detector);
  runManager->SetUserInitialization(new PSIPhysicsList);
  
//#ifdef G4VIS_USE
  // visualization manager
  //G4VisManager* visManager = new G4VisExecutive;
  //visManager->Initialize();
//#endif

 G4UIsession* session=0;
  
  if (argc==1)   // Define UI session for interactive mode.
    {
      // G4UIterminal is a (dumb) terminal.
#if defined(G4UI_USE_XM)
      session = new G4UIXm(argc,argv);
#elif defined(G4UI_USE_WIN32)
      session = new G4UIWin32();
#elif defined(G4UI_USE_TCSH)
      session = new G4UIterminal(new G4UItcsh);      
#else
      session = new G4UIterminal();
#endif
    }
  
#ifdef G4VIS_USE
  // visualization manager
  G4VisManager* visManager = new G4VisExecutive;
  visManager->Initialize();
#endif

  
    // set user action classes
  runManager->SetUserAction(new PSIPrimaryGeneratorAction(detector));
  runManager->SetUserAction(new PSIRunAction);
  PSIEventAction* eventaction = new PSIEventAction;
  runManager->SetUserAction(eventaction);
  //runManager->SetUserAction(new PSIStackingAction);
  runManager->SetUserAction(new PSISteppingAction(detector, eventaction));

  //Initialize G4 kernel
  runManager->Initialize();
    
 
  // get the pointer to the User Interface manager 
  G4UImanager* UI = G4UImanager::GetUIpointer();  

  if (session)   // Define UI session for interactive mode.
    {
      // G4UIterminal is a (dumb) terminal.
      UI->ApplyCommand("/control/execute vis.mac");    
#if defined(G4UI_USE_XM) || defined(G4UI_USE_WIN32)
      // Customize the G4UIXm,Win32 menubar with a macro file :
      UI->ApplyCommand("/control/execute visTutor/gui.mac");
#endif
      session->SessionStart();
      delete session;
    }
  else           // Batch mode
    { 
      G4String command = "/control/execute ";
      G4String fileName = argv[1];
      UI->ApplyCommand(command+fileName);
    }
    
       
  // job termination
#ifdef G4VIS_USE
  delete visManager;
#endif
  delete runManager;

  return 0;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
