//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// Id: PSIDetectorConstruction.cc,v 1.21 2006/01/14 10:12:08 Volnykh V.P. Exp 
// GEANT4 tag $Name: v2-05 $
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "PSIDetectorConstruction.hh"
//#include "PSIDetectorMessenger.hh"

#include "G4Material.hh"
#include "G4Box.hh"
#include "G4LogicalVolume.hh"
#include "G4PVPlacement.hh"
#include "G4PVReplica.hh"
#include "G4UniformMagField.hh"

#include "G4GeometryManager.hh"
#include "G4PhysicalVolumeStore.hh"
#include "G4LogicalVolumeStore.hh"
#include "G4SolidStore.hh"

#include "G4VisAttributes.hh"
#include "G4Colour.hh"

// VGM demo
#include "Geant4GM/volumes/Factory.h"
#include "RootGM/volumes/Factory.h"
#include "TGeoManager.h"
#include "XmlVGM/GDMLExporter.h"

#include "G4LogicalBorderSurface.hh"
#include "G4LogicalSkinSurface.hh"
#include "G4OpticalSurface.hh"

// end VGM demo

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PSIDetectorConstruction::PSIDetectorConstruction()
  : logicWorld(0)
{}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PSIDetectorConstruction::~PSIDetectorConstruction()
{}

  // ---------------------------------------------------------------------------
  // VGM demo 
  //

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4LogicalVolume*  
PSIDetectorConstruction::FindVolume(const G4String& name) const
{
  G4LogicalVolumeStore* lvStore = G4LogicalVolumeStore::GetInstance();
  
  for (G4int i=0; i<G4int(lvStore->size()); i++) {
    G4LogicalVolume* lv = (*lvStore)[i];
    if (lv->GetName() == name) return lv;
  }

  G4String text = "PSIDetectorConstruction:: FindVolume:\n"; 
  text = text + "    Logical volume " + name + " does not exist.";
  std::cerr << text << G4endl;
  return 0;
}                    

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

G4VPhysicalVolume* PSIDetectorConstruction::Construct()
{
  // 
  // Import geometry from Root
  //

  // Import geometry from the root file
  new TGeoManager("PSI", "Geant4 PSI pibeta");
  gGeoManager->Import("detector_G4.root");

  // Import geometry from Root to VGM
  RootGM::Factory rtFactory;
  rtFactory.SetDebug(1);
  rtFactory.Import(gGeoManager->GetTopVolume());

  // Export VGM geometry to Geant4
  //
  Geant4GM::Factory g4Factory;
  g4Factory.SetDebug(1);
  rtFactory.Export(&g4Factory);
  G4VPhysicalVolume* world = g4Factory.World();
  
  //Geant4GM::Factory g4factory;
  //factory.Import(physiWorld);
  
  //XmlVGM::GDMLExporter xmlExporter2(&g4Factory); 
  //xmlExporter2.GenerateXMLGeometry();
  
  //....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
  
  

  
    
  // Get volumes from logical volume store by name
  

  logicDEGR = FindVolume("DEGR");
  logicTUBE = FindVolume("TUBE");  
  //logicTBI1 = FindVolume("TBI1");
  //logicTBI2 = FindVolume("TBI2");
  //logicTBI3 = FindVolume("TBI3");
  //logicTBI4 = FindVolume("TBI4");
  //logicTBO1 = FindVolume("TBO1");
  //logicTBO2 = FindVolume("TBO2");
  //logicTBO3 = FindVolume("TBO3");
  //logicTBO4 = FindVolume("TBO4");
  logicMWPC = FindVolume("MWPC");
  logicPVET = FindVolume("PVET");
  logicPV01 = FindVolume("PV01"); 
  logicPV02 = FindVolume("PV02");
  logicPV03 = FindVolume("PV03");
  logicPV04 = FindVolume("PV04");
  logicPV05 = FindVolume("PV05");
  logicPV06 = FindVolume("PV06");
  logicPV07 = FindVolume("PV07");
  logicPV08 = FindVolume("PV08");
  logicPV09 = FindVolume("PV09");
  logicPV10 = FindVolume("PV10");
  logicPV11 = FindVolume("PV11");
  logicPV12 = FindVolume("PV12");
  logicPV13 = FindVolume("PV13");
  logicPV14 = FindVolume("PV14");
  logicPV15 = FindVolume("PV15");
  logicPV16 = FindVolume("PV16");
  logicPV17 = FindVolume("PV17");
  logicPV18 = FindVolume("PV18");
  logicPV19 = FindVolume("PV19");
  logicPV20 = FindVolume("PV20");


                    
  // Visualization attributes
  //
  logicWorld = world->GetLogicalVolume();
  logicWorld->SetVisAttributes (G4VisAttributes::Invisible);
  G4VisAttributes* simpleBoxVisAtt= new G4VisAttributes(G4Colour(0.0,1.0,0.0));
  simpleBoxVisAtt->SetVisibility(true);
  //logicMWPC->SetVisAttributes(simpleBoxVisAtt);
  logicTUBE->SetVisAttributes(simpleBoxVisAtt);
  logicDEGR->SetVisAttributes(simpleBoxVisAtt);
  //logicTBI1->SetVisAttributes(simpleBoxVisAtt);
  //logicTBI2->SetVisAttributes(simpleBoxVisAtt);
  //logicTBI3->SetVisAttributes(simpleBoxVisAtt);
  //logicTBI4->SetVisAttributes(simpleBoxVisAtt);
  //logicTBO1->SetVisAttributes(simpleBoxVisAtt);
  //logicTBO2->SetVisAttributes(simpleBoxVisAtt);
  //logicTBO3->SetVisAttributes(simpleBoxVisAtt);
  //logicTBO4->SetVisAttributes(simpleBoxVisAtt);
  //logicMWPC->SetVisAttributes(simpleBoxVisAtt);

  //logicPVET->SetVisAttributes(simpleBoxVisAtt);
  //logicAbsorber->SetVisAttributes(simpleBoxVisAtt);
  //logicGap->SetVisAttributes(simpleBoxVisAtt);
  
  return world;

  //
  // end VGM demo
  //---------------------------------------------------------------------------
}
