//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
//
//
// Id: PSIPrimaryGeneratorAction.cc,v 1.7 2006/01/15 15:38:18 Volnykh V.P. Exp 
// GEANT4 tag Name: geant4-08-00 
//
// 

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

#include "PSIPrimaryGeneratorAction.hh"

#include "PSIDetectorConstruction.hh"
#include "PSIPrimaryGeneratorMessenger.hh"
#include "PSIAnalysis.hh"

#include "G4Event.hh"
#include "G4ParticleGun.hh"
#include "G4ParticleTable.hh"
#include "G4ParticleDefinition.hh"
#include "G4RunManager.hh"
#include "Randomize.hh"

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PSIPrimaryGeneratorAction::PSIPrimaryGeneratorAction(
                                             PSIDetectorConstruction* PSIDC)
:PSIDetector(PSIDC),rndmFlag("off")
{

  G4int n_particle = 1;
  particleGun  = new G4ParticleGun(n_particle);
  
  //create a messenger for this class
  gunMessenger = new PSIPrimaryGeneratorMessenger(this);

  // default particle kinematic

  G4ParticleTable* particleTable = G4ParticleTable::GetParticleTable();
  G4String particleName;
  G4ParticleDefinition* particle
                    = particleTable->FindParticle(particleName="e-");
  particleGun->SetParticleDefinition(particle);
  particleGun->SetParticleMomentumDirection(G4ThreeVector(1.,0.,0.));
  particleGun->SetParticleEnergy(50.*MeV);






  //G4double position = -0.5*(PSIDetector->GetWorldSizeX());
  //particleGun->SetParticlePosition(G4ThreeVector(position,0.*cm,0.*cm));
  particleGun->SetParticlePosition(G4ThreeVector(0.*cm,0.*cm,0.*cm));

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

PSIPrimaryGeneratorAction::~PSIPrimaryGeneratorAction()
{
  delete particleGun;
  delete gunMessenger;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

void PSIPrimaryGeneratorAction::GeneratePrimaries(G4Event* anEvent)
{
  //this function is called at the begining of event
  // 
/*
  G4double x0 = -0.5*(PSIDetector->GetWorldSizeX());
  G4double y0 = 0.*cm, z0 = 0.*cm;
  if (rndmFlag == "on")
     {y0 = (PSIDetector->GetCalorSizeYZ())*(G4UniformRand()-0.5);
      z0 = (PSIDetector->GetCalorSizeYZ())*(G4UniformRand()-0.5);
     }
  particleGun->SetParticlePosition(G4ThreeVector(x0,y0,z0));
*/
  G4double RANDOM,r;

   RANDOM = 8194*G4UniformRand();

  if (RANDOM < 20.0) {
    r = 112.0+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 45.0) {
    r = 112.1+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 74.0) {
    r = 112.2+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 109.0) {
    r = 112.3+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 193.0) {
    r = 112.4+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 311.0) {
    r = 112.5+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 501.0) {
    r = 112.6+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 821.0) {
    r = 112.7+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 1232.0) {
    r = 112.8+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 1662.0) {
    r = 112.9+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 2172.0) {
    r = 113.0+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 2732.0) {
    r = 113.1+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 3317.0) {
    r = 113.2+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 3885.0) {
    r = 113.3+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 4445.0) {
    r = 113.4+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 5012.0) {
    r = 113.5+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 5582.0) {
    r = 113.6+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 6122.0) {
    r = 113.7+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 6612.0) {
    r = 113.8+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 7076.0) {
    r = 113.9+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 7466.0) {
    r = 114.0+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 7786.0) {
    r = 114.1+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 8006.0) {
    r = 114.2+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 8129.0) {
    r = 114.3+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 8173.0) {
    r = 114.4+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
  if (RANDOM < 8194.0) {
    r = 114.5+0.1*G4UniformRand();
    primaryParticleEnergy = std::sqrt(r*r+19479.83515)-139.57018;} else
    particleGun->SetParticleEnergy(0.*MeV);

   // G4double partEnergy = particleGun->GetParticleEnergy();

    //PSIAnalysis::getInstance()->addPrimaryEnergy(edep);

    particleGun->SetParticleEnergy(primaryParticleEnergy);
    G4ThreeVector ddir = particleGun->GetParticlePosition();

    G4double xx = ddir.x();
    G4double yy = ddir.y();
    G4double zz = ddir.z();
    


    PSIAnalysis* analysis = PSIAnalysis::getInstance();
    analysis->analysePrimaryGenerator(primaryParticleEnergy,xx,yy,zz);


  particleGun->GeneratePrimaryVertex(anEvent);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo......

